var PuzzleGame = new Class({
  extend: {
    dimensions: {
      small:  { width:  616, height: 385 },
      medium: { width:  880, height: 550 },
      big:    { width: 1056, height: 660 }
    },
    STAGE: document.getElementById('stage'),
    currentSize: null,

    determineFixedSize: function() {
      if ( $(window).size().x <= 920 || $(window).size().y <= 590 ) {
        PuzzleGame.currentSize = 'small';
      } else if ( $(window).size().x <= 1096 || $(window).size().y <= 700 ) {
        PuzzleGame.currentSize = 'medium';
      } else {
        PuzzleGame.currentSize = 'big';
      }
    }
  },
  // initialize {{{
  initialize: function() {
    var this_ = this;

    PuzzleGame.determineFixedSize();

    if (window.location.search.match(/new-window/)) {
      this.openedInNewWindow = true;
    }

    this.soundEffect = new PuzzleGame.SoundEffect({
      game: this
    });
    this.muted = false;

    this.stage = PuzzleGame.STAGE;

    if ( !PuzzleGame.Actor.STYLESHEET_SUPPORT ) {
      this.setFixedSizeStyles();
    }
  },
  // }}}
  // setupGame {{{
  setupGame: function() {
    this.setupHomeScreen();
  },
  // }}}
  // toggleMute {{{
  toggleMute: function() {
    if (this.muted) {
      this.muted = false;

      this.soundButton.changeStyle({ filename: 'sound' });
    } else {
      this.muted = true;
      this.soundEffect.stop();
      this.soundButton.changeStyle({ filename: 'sound-mute' });
    }
  },
  // }}}
  // setupHomeScreen {{{
  setupHomeScreen: function(opts) {
    var this_ = this;
    var pad = function(number) {
      return (number < 10 ? '0' : '') + number
    }

    // Stop the sound if going back to home screen and a sound is still playing
    game.soundEffect.stop();

    // If sound button has already been created, we juste move it
    if (this.soundButton != undefined) {
      this.soundButton.setStyles({ left: '13%', bottom: '0%' });
    } else {
      // If the sound button doesn't exist, we create it !
      this.soundButton = new PuzzleGame.Actor({
        filename: this.muted ? 'sound-mute' : 'sound',
        path: 'assets/images/buttons',
        styles: { left: '13%', bottom: '0%' },
        click: this.toggleMute.bind(this)
      });
    }

    this.currentSection = null;

    $(document.body).removeClass('section-inside').removeClass('section-outside').removeClass('section-water');

    // Set variables so if the puzzle set is completed, we can change the images

    // If we receive options, it means we are done with all the puzzles in a set
    // The number is received as an option
    if (opts != undefined) {
      switch(opts) {
        case '12':
          Config.filename_12_completed = '-completed';
          break;

        case '16':
          Config.filename_16_completed = '-completed';
          break;

        case '20':
          Config.filename_20_completed = '-completed';
          break;
      }
    }

    this.stageElements = [

      // Background
      new PuzzleGame.Actor({
        filename: 'home',
        extension: 'jpg',
        path: 'assets/images/backgrounds/',
        styles: { left: '0%', top: '0%' },
        dimensions: PuzzleGame.dimensions,
        interactive: false
      }),

      // Meta buttons
      new PuzzleGame.Actor({
        filename: 'credit',
        filenameHover :'credit-hover',
        path: 'assets/images/buttons',
        styles: { left: '3%', bottom: '0%' },
        click: this.showCredits.bind(this)
      }),

      new PuzzleGame.Actor({
        filename: 'help',
        filenameHover :'help-hover',
        path: 'assets/images/buttons',
        styles: { left: '8%', bottom: '0%' },
        click: this.showHelp.bind(this)
      }),

      // game buttons
      new PuzzleGame.Actor({
        filename: 'pieces-number-12' + Config.filename_12_completed,
        filenameHover :'pieces-number-12' + Config.filename_12_completed + '-hover',
        path: 'assets/images/buttons',
        styles: { left: '11%', top: '25%' },
        click: function() {
          this_.destroy();
          this_.setupPuzzlesSection('12');
          $(document.body).addClass('section-puzzles puzzle-12');
        }
      }),

      new PuzzleGame.Actor({
        filename: 'pieces-number-16' + Config.filename_16_completed,
        filenameHover :'pieces-number-16' + Config.filename_16_completed + '-hover',
        path: 'assets/images/buttons',
        styles: { left: '11%', top: '43%' },
        click: function() {
          this_.destroy();
          this_.setupPuzzlesSection('16');
          $(document.body).addClass('section-puzzles puzzle-16');
        }
      }),

      new PuzzleGame.Actor({
        filename: 'pieces-number-20' + Config.filename_20_completed,
        filenameHover :'pieces-number-20' + Config.filename_20_completed + '-hover',
        path: 'assets/images/buttons',
        styles: { left: '11%', top: '61%' },
        click: function() {
          this_.destroy();
          this_.setupPuzzlesSection('20');
          $(document.body).addClass('section-puzzles puzzle-20');
        }
      })
    ]

    if (this.openedInNewWindow) {
      this.stageElements.push(
        new PuzzleGame.Actor({
          filename: 'close-dialog',
          filenameHover: 'close-dialog-hover',
          path: 'assets/images/buttons',
          styles: { right: '2%', bottom: '0%' },
          click: function() {
            window.close();
          }
        })
      );
    }
  },
  // }}}
  // destroy {{{
  destroy: function() {
    this.stageElements.each('destroy');
  },
  // }}}

  // showPuzzleCompletedWindow {{{
  showCompletedPuzzleWindow: function() {
    var this_ = this;

    this.hideDialog();

    this.veil = new PuzzleGame.StageElement({
      element:  $E('div')._,
      className: 'veil',
      interactive: false
    });

    this.dialog = new PuzzleGame.Actor({
      filename: 'completed-puzzle-bg',
      path: 'assets/images/dialogs',
      className: 'dialog completed-puzzle-bg-dialog',
      interactive: false,
      autoAppend: false,
      children: [
        new PuzzleGame.Actor({
          filename: 'completed-puzzle-arrow',
          path: 'assets/images/buttons',
          filenameHover: 'completed-puzzle-arrow',
          className: 'dialog-button completed-puzzle-arrow-button',
          autoAppend: false,
          click: function() {
            this_.puzzle.switchPuzzle(1);
            this_.hideDialog();
          }
        })
      ]
    });

    PuzzleGame.OnloadObserver.queueHandler((function() {
      $(PuzzleGame.STAGE).append(this.dialog.element);
    }).bind(this));
  },
  // }}}

  // showPuzzleCompletedSetWindow {{{
  showCompletedPuzzleSetWindow: function() {
    var this_ = this;

    this.hideDialog();

    this.veil = new PuzzleGame.StageElement({
      element:  $E('div')._,
      className: 'veil',
      interactive: false
    });

    this.dialog = new PuzzleGame.Actor({
      filename: 'completed-puzzle-set-bg',
      path: 'assets/images/dialogs',
      className: 'dialog completed-puzzle-set-bg-dialog',
      interactive: false,
      autoAppend: false
    });

    PuzzleGame.OnloadObserver.queueHandler((function() {
      $(PuzzleGame.STAGE).append(this.dialog.element);
    }).bind(this));
  },
  // }}}

  // showConfirmation {{{
  showConfirmation: function() {
    this.hideDialog();

    this.veil = new PuzzleGame.StageElement({
      element:  $E('div')._,
      className: 'veil',
      interactive: false
    });

    this.dialog = new PuzzleGame.Actor({
      filename: 'confirmation',
      path: 'assets/images/dialogs',
      className: 'dialog home-screen-dialog help-dialog',
      interactive: false,
      autoAppend: false,
      children: [
        new PuzzleGame.Actor({
          filename: 'close-dialog',
          path: 'assets/images/buttons',
          filenameHover: 'close-dialog-hover',
          className: 'dialog-button close-dialog-button',
          autoAppend: false,
          click: this.hideDialog.bind(this)
        }),

        new PuzzleGame.Actor({
          filename: 'close-dialog',
          path: 'assets/images/buttons',
          filenameHover: 'close-dialog-hover',
          className: 'accept-button accept-button',
          autoAppend: false,
          click: function(){
            window.close();
          }
        })
      ]
    });

    PuzzleGame.OnloadObserver.queueHandler((function() {
      $(PuzzleGame.STAGE).append(this.dialog.element);
    }).bind(this));
  },
  // }}}

  showResetConfirmation: function() {
    var this_ = this;

    this.hideDialog();

    this.veil = new PuzzleGame.StageElement({
      element:  $E('div')._,
      className: 'veil',
      interactive: false
    });

    this.dialog = new PuzzleGame.Actor({
      filename: 'reset-dialog',
      path: 'assets/images/dialogs',
      className: 'dialog home-screen-dialog reset-dialog',
      interactive: false,
      autoAppend: false,
      children: [
        new PuzzleGame.Actor({
          filename: 'close-dialog',
          path: 'assets/images/buttons',
          filenameHover: 'close-dialog-hover',
          className: 'dialog-button close-dialog-button',
          autoAppend: false,
          click: this.hideDialog.bind(this)
        }),

        new PuzzleGame.Actor({
          filename: 'reset-confirmation-button',
          path: 'assets/images/buttons',
          filenameHover: 'reset-confirmation-button-hover',
          className: 'reset-puzzle-button',
          autoAppend: false,
          click: function() {
            this_.puzzle.resetPuzzle();
            this_.hideDialog();
          }
        })
      ]
    });

    PuzzleGame.OnloadObserver.queueHandler((function() {
      $(PuzzleGame.STAGE).append(this.dialog.element);
    }).bind(this));
  },

  // showHelp {{{
  showHelp: function() {
    this.hideDialog();

    this.veil = new PuzzleGame.StageElement({
      element:  $E('div')._,
      className: 'veil',
      interactive: false
    });

    this.dialog = new PuzzleGame.Actor({
      filename: 'help-dialog',
      path: 'assets/images/dialogs',
      className: 'dialog home-screen-dialog help-dialog',
      interactive: false,
      autoAppend: false,
      children: [
        new PuzzleGame.Actor({
          filename: 'close-dialog',
          path: 'assets/images/buttons',
          filenameHover: 'close-dialog-hover',
          className: 'dialog-button close-dialog-button',
          autoAppend: false,
          click: this.hideDialog.bind(this)
        })
      ]
    });

    PuzzleGame.OnloadObserver.queueHandler((function() {
      $(PuzzleGame.STAGE).append(this.dialog.element);
    }).bind(this));
  },
  // }}}
  // showCredits {{{
  showCredits: function() {
    this.hideDialog();

    this.veil = new PuzzleGame.StageElement({
      element:  $E('div')._,
      className: 'veil',
      interactive: false
    });

    this.dialog = new PuzzleGame.Actor({
      filename: 'credit-dialog',
      path: 'assets/images/dialogs',
      className: 'dialog home-screen-dialog credit-dialog',
      interactive: false,
      autoAppend: false,
      children: [
        new PuzzleGame.Actor({
          filename: 'close-dialog',
          path: 'assets/images/buttons',
          filenameHover: 'close-dialog-hover',
          className: 'dialog-button close-dialog-button',
          autoAppend: false,
          click: this.hideDialog.bind(this)
        })
      ]
    });

    PuzzleGame.OnloadObserver.queueHandler((function() {
      $(PuzzleGame.STAGE).append(this.dialog.element);
    }).bind(this));
  },
  // }}}
  // setupPuzzlesSection {{{
  setupPuzzlesSection: function(pieceNumber) {
    var this_ = this;

    // Set the currentPuzzlePieceCount
    this.currentPuzzlePieceCount = pieceNumber;

    // Play sound and change the position of sound button
    game.soundEffect.play('distribute');
    this.soundButton.setStyles({ left: '8%', bottom: '0%' });

    // Define puzzle
    this.puzzle = new PuzzleGame.Puzzle;

    this.stageElements = [

      // Back home button
      new PuzzleGame.Actor({
        filename: 'home',
        filenameHover :'home-hover',
        path: 'assets/images/buttons',
        styles: { left: '3%', bottom: '0%' },
        click: function() {
          this_.destroy();
          this_.setupHomeScreen();

          // Delete all pieces that are on stage
          Config.PUZZLES[this_.currentPuzzlePieceCount][game.puzzle.currentPuzzleId - 1].pieceBox = game.puzzle.a_puzzlePiece;
          game.puzzle.a_puzzlePiece.each('destroy')
        }
      }),

      // Reset puzzle
      new PuzzleGame.Actor({
        filename: 'reset',
        filenameHover :'reset',
        path: 'assets/images/buttons',
        styles: { right: '10%', bottom: '0%' },
        click: function() {
          this_.showResetConfirmation();
        }
      }),

      // Clue
      new PuzzleGame.Actor({
        filename: 'clue-button',
        filenameHover :'clue-button',
        path: 'assets/images/buttons',
        styles: { right: '3%', bottom: '0%' },
        mouseenter: function() { this_.puzzle.showClue(); },
        mouseleave: function() { this_.puzzle.hideClue(); }
      }),

      // Background for active pieces
      new PuzzleGame.Actor({
        filename: 'piece-number',
        extension: 'png',
        path: 'assets/images/backgrounds/' + this_.currentPuzzlePieceCount,
        styles: { zIndex: 405, left: '37.3%', bottom: '0%' },
        dimensions: PuzzleGame.dimensions,
        interactive: false
      })
    ];

    // Make sure there is more than 1 puzzle for the current size
    if (Config.PUZZLES[this.currentPuzzlePieceCount].length > 1) {
      this.stageElements.push(
        // Previous/Next button
        new PuzzleGame.Actor({
          filename: 'previous',
          filenameHover: 'previous-hover',
          path: 'assets/images/buttons',
          styles: { zIndex: 406, left: '38.7%', bottom: '2%' },
          click: function() {
            if (this_.puzzle.arePiecesDoneBeingPlaced == true) {
              this_.puzzle.switchPuzzle(-1);
            }
          }
        }),

        new PuzzleGame.Actor({
          filename: 'next',
          filenameHover: 'next-hover',
          path: 'assets/images/buttons',
          styles: { zIndex: 406, right: '39.6%', bottom: '2%' },
          click: function() {
            if (this_.puzzle.arePiecesDoneBeingPlaced == true) {
              this_.puzzle.switchPuzzle(1);
            }
          }
        })
      );
    }

    // Puzzles background
    this.bg = new PuzzleGame.Actor({
      filename: 'background',
      extension: 'jpg',
      path: 'assets/images/backgrounds/' + this_.currentPuzzlePieceCount,
      styles: { zIndex: 399, left: '0%', top: '0%' },
      dimensions: PuzzleGame.dimensions,
      interactive: false
    });

    this.stageElements.push(this.bg);

    // Setup the puzzle
    this_.puzzle.setupPuzzle();
  },
  // }}}
  // hideDialog {{{
  hideDialog: function() {
    this.dialog && this.dialog.destroy();
    this.dialog = null;
    this.veil && this.veil.destroy();
    this.veil = null;
  },
  // }}}
  // setupSection {{{
  setupSection: function(sectionName) {
    this.soundEffect.stop();
    this.currentSection = new PuzzleGame.Section({
      sectionName: sectionName,
      sectionConfig: Config.SECTIONS[ sectionName ],
      game: this
    });
    this.soundButton.setStyles({ left: '7%', bottom: '0%' });
  },
  // }}}
  // setFixedSizeStyles {{{
  setFixedSizeStyles: function() {
    var marginTop, marginLeft;
    if ( PuzzleGame.currentSize == 'small' ) {

      marginTop  = -202;
      marginLeft = -318;
      this.stage.style.fontSize = '0.7em';

    } else if ( PuzzleGame.currentSize == 'medium' ) {

      marginTop  = -285;
      marginLeft = -450;
      this.stage.style.fontSize = '1em';

    } else {

      marginTop  = -340;
      marginLeft = -538;
      this.stage.style.fontSize = '1.2em';
    }

    $(document.body).addClass( PuzzleGame.currentSize );

    $(PuzzleGame.STAGE).setStyle({
      width:  PuzzleGame.dimensions[PuzzleGame.currentSize].width,
      height: PuzzleGame.dimensions[PuzzleGame.currentSize].height,
      marginTop:  marginTop,
      marginLeft: marginLeft
    });
  }
  // }}}

});
