PuzzleGame.Actor = new Class(PuzzleGame.StageElement, {

  extend: {
    path: 'assets/images/',
    STYLESHEET_SUPPORT: document.styleSheets[0].insertRule !== undefined
  },

  initialize: function(opts) {
    var this_ = this;

    PuzzleGame.OnloadObserver.incrementQueue();

    if (PuzzleGame.Actor.STYLESHEET_SUPPORT) {
      PuzzleGame.determineFixedSize();
    }

    this.filename = opts['filename'];
    this.filenameHover = opts['filenameHover'] || this.filename;

    this.element = document.createElement('div');
    // this.objectElement = $(this.element)

    this.id = this.element.id = this.filename;
    this.element.className = 'actor ' + this.filename;
    this.element.unselectable = 'on';

    this.position = opts['position'];

    // StageElement handles the position
    this.$super(opts);

    this.path = opts['path'] || PuzzleGame.Actor.path;
    this.extension = opts['extension'] || 'png';

    var currentSizeName = PuzzleGame.currentSize;

    var img = new Image;

    img.onload = function() {
      PuzzleGame.OnloadObserver.decrementQueue();

      var currentWidth  = img.width,
        currentHeight = img.height;

      var small = PuzzleGame.dimensions['small']['height'],
        medium  = PuzzleGame.dimensions['medium']['height'],
        big     = PuzzleGame.dimensions['big']['height'];

      switch (currentSizeName) {
        case 'small':
          this_.dimensions = {
            small:  { width:  currentWidth,                        height:  currentHeight },
            medium: { width: (currentWidth * medium/small).ceil(), height: (currentHeight * medium/small).ceil() },
            big:    { width: (currentWidth * big/small).ceil(),    height: (currentHeight * big/small).ceil() }
          };
        break;
        case 'medium':
          this_.dimensions = {
            small:  { width: (currentWidth * small/medium).ceil(), height: (currentHeight * small/medium).ceil() },
            medium: { width:  currentWidth,                        height:  currentHeight },
            big:    { width: (currentWidth * big/medium).ceil(),   height: (currentHeight * big/medium).ceil() }
          };
        break;
        case 'big':
          this_.dimensions = {
            small:  { width: (currentWidth * small/big).ceil(),  height: (currentHeight * small/big).ceil() },
            medium: { width: (currentWidth * medium/big).ceil(), height: (currentHeight * medium/big).ceil() },
            big:    { width:  currentWidth,                      height:  currentHeight }
          };
        break;
      }


      this_.drawImage();

      img.onload = function() {
        img.onload = img = null;
      };
      img.src = this_.path + '/' + currentSizeName + '/' + this_.filenameHover + '.' + this_.extension;
    };
    img.src = this_.path + '/' + currentSizeName + '/' + this_.filename + '.' + this_.extension;
  },

  drawImage: function() {
    if (PuzzleGame.Actor.STYLESHEET_SUPPORT ) {
      this.canHasStyle = true
      this.styleElement = document.createElement('style');
      $(document.getElementsByTagName('head')[0]).append( this.styleElement );

      var styleSheet = document.styleSheets[ document.styleSheets.length - 1 ];

      var cssRules = [], hasHoverState = this.filenameHover !== this.filename;

      cssRules.push(
        '@media screen and (max-width: 919px), screen and (max-height: 589px) {' +
          '#' + this.id + ' {' +
            'width:  ' + this.dimensions['small'].width  + 'px;' +
            'height: ' + this.dimensions['small'].height + 'px;' +
            'background-image: url(' + this.path + '/small/' + this.filename + '.' + this.extension + ');' +
          '}' + (
            hasHoverState ?
            '#' + this.id + '.interactive:hover, #' + this.id + '.interactive.highlight,' +
            '.interactive:hover > #' + this.id + '.child {' +
              'background-image: url(' + this.path + '/small/' + this.filenameHover + '.' + this.extension + ');' +
            '}' : ''
          ) +
        '}'
      );

      cssRules.push(
        '@media screen and (min-width: 920px) and (min-height: 590px) and (max-width: 1095px), ' +
            'screen and (min-width: 920px) and (min-height: 590px) and (max-height: 699px) {' +
          '#' + this.id + ' {' +
            'width:  ' + this.dimensions['medium'].width  + 'px;' +
            'height: ' + this.dimensions['medium'].height + 'px;' +
            'background-image: url(' + this.path + '/medium/' + this.filename + '.' + this.extension + ');' +
          '}' + (
            hasHoverState ?
            '#' + this.id + '.interactive:hover, #' + this.id + '.interactive.highlight,' +
            '.interactive:hover > #' + this.id + '.child {' +
              'background-image: url(' + this.path + '/medium/' + this.filenameHover + '.' + this.extension + ');' +
            '}' : ''
          ) +
        '}'
      );

      cssRules.push(
        '@media screen and (min-width: 1096px) and (min-height: 700px) {' +
          '#' + this.id + ' {' +
            'width:  ' + this.dimensions['big'].width  + 'px;' +
            'height: ' + this.dimensions['big'].height + 'px;' +
            'background-image: url(' + this.path + '/big/' + this.filename + '.' + this.extension + ');' +
          '}' + (
            hasHoverState ?
            '#' + this.id + '.interactive:hover, #' + this.id + '.interactive.highlight,' +
            '.interactive:hover > #' + this.id + '.child {' +
              'background-image: url(' + this.path + '/big/' + this.filenameHover + '.' + this.extension + ');' +
            '}' : ''
          ) +
        '}'
      );

      cssRules.each(function(rule, index) {
        styleSheet.insertRule(rule, index);
      });

    } else {
      var this_ = this;
      this.canHasStyle = false
      $(this.element).setStyle({
        width:  this_.dimensions[PuzzleGame.currentSize].width,
        height: this_.dimensions[PuzzleGame.currentSize].height,
        background: 'url(' + this_.path + '/' + PuzzleGame.currentSize + '/' + this_.filename + '.' + this_.extension + ')'
      }).on({
        mouseover: function() {
          $(this).setStyle('background', 'url(' + this_.path + '/' + PuzzleGame.currentSize + '/' + this_.filenameHover + '.' + this_.extension + ')');
        },
        mouseout: function() {
          if(!$(this).hasClass('clicked'))
            $(this).setStyle('background', 'url(' + this_.path + '/' + PuzzleGame.currentSize + '/' + this_.filename + '.' + this_.extension + ')');
        },
        click: function() {
          if($(this).hasClass('big-circle'))
             $(this).setStyle('background', 'url(' + this_.path + '/' + PuzzleGame.currentSize + '/' + this_.filenameHover + '.' + this_.extension + ')');
        }
      });
    }
  },

  triggerHighlight: function() {
    var this_ = this
    $(this.element).setStyle('background', 'url(' + this_.path + '/' + PuzzleGame.currentSize + '/' + this_.filenameHover + '.' + this_.extension + ')');
  },

  changeStyle: function(opts) {
    if ( this.styleElement ) {
      $(this.styleElement).remove();
    }
    this.filename = opts['filename'];
    this.filenameHover = opts['filenameHover'] || this.filename;
    this.path = opts['path'] || this.path || PuzzleGame.Actor.path;
    this.extension = opts['extension'] || this.extension || 'png';
    this.drawImage();
  },

  destroy: function() {
    this.$super();
    if ( this.styleElement ) {
      $(this.styleElement).remove();
    }
  }

});
