/*
  * Author : Luc-Olivier Boulet
  * iXmedia - www.ixmedia.com
*/
PuzzleGame.Puzzle = new Class({

  // =============================================
  // initialize
  initialize: function(opts) {
    var this_ = this;

    this.currentPuzzleId      = 1; // Current visible puzzle ID
    this.currentPuzzleIdName  = 'puzzle' + this.currentPuzzleId;
    this.oldDimensions        = PuzzleGame.currentSize;

    this.a_puzzlePiece                = [];
    this.a_stagePadding               = [];
    this.watermarkposition            = { left: 0, bottom: 0 };
    this.cluePosition                 = { left: 0, bottom: 0 };
    this.currentPiecePositionOnBefore = null;
    this.currentPiece                 = null;

    this.arePiecesDoneBeingPlaced     = false;

    this.draggableOptions = {
      onStop:   function() {this_.detectIfPieceIsNearTarget(); },
      onBefore: function() {
        // Fix for IE
        setTimeout(function(){
          this_.currentPiecePositionOnBefore = [this_.currentPiece.element.offsetTop, this_.currentPiece.element.offsetLeft];
        }, 0)
      }
    }

    // Define stage padding depending on current size
    switch(PuzzleGame.currentSize) {
      case 'big' :    this_.a_stagePadding = [210, 160]; break;
      case 'medium' : this_.a_stagePadding = [175, 140]; break;
      case 'small' :  this_.a_stagePadding = [150, 135]; break;
    }
  },

  // =============================================
  // setupPuzzle
  setupPuzzle: function() {
    this.initWatermark();
    this.initPieces();
  },

  // =============================================
  // initWatermark
  initWatermark: function() {
    var this_ = this;

    // Get the good filename, position for watermark and for the clue
    switch (game.currentPuzzlePieceCount) {
      case '12':
        this.watermarkposition  = { left: '30.4%', bottom: '33%' };
        this.cluePosition       = { left: '30.4%', bottom: '33%' };
        break;

      case '16':
        this.watermarkposition  = { left: '31%', bottom: '31%' };
        this.cluePosition       = { left: '31%', bottom: '31%' };
        break;

      case '20':
        this.watermarkposition  = { left: '35%', bottom: '19%' };
        this.cluePosition       = { left: '35%', bottom: '19%' };
        break;

      default:
        this.watermarkposition  = { left: '30.4%', bottom: '33%' };
        this.cluePosition       = { left: '30.4%', bottom: '33%' };
        break;
    }

    // Create the watermark actor
    this.watermark = new PuzzleGame.Actor({
      filename: 'watermark',
      extension: 'png',
      path: 'assets/images/backgrounds/' + game.currentPuzzlePieceCount,
      styles: {
        zIndex: 402,
        left: this_.watermarkposition.left,
        bottom: this_.watermarkposition.bottom
      },
      dimensions: PuzzleGame.dimensions,
      interactive: false
    })

    // Add the watermark to stageElements
    game.stageElements.push(this.watermark);
  },

  // =============================================
  // initPieces
  initPieces: function() {
    var this_ = this;

    var path              = '',
        draggable         = null,
        piecePosition     = null,
        currentTempPiece  = null,
        newPosition       = null;

    if (Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].isDone == true) {
      game.soundEffect.play(Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].sound);
    }

    // Loop on every pieces
    for (var i = 1, l = game.currentPuzzlePieceCount; i <= l; i++) {
      path = 'assets/images/puzzles/' + game.currentPuzzlePieceCount + '/puzzle-' + this.currentPuzzleId;

      // Check if pieceBox is defined or not
      if (Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieceBox != undefined) {

      // If Yes, it means we've already passed in here for the current puzzle, so we are gonna get the position already defined
        piecePosition = {
          zIndex: 403,
          top:    Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces[i -1].actualPosition[0] + 'px',
          left:   Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces[i -1].actualPosition[1] + 'px'
        };

        this_.arePiecesDoneBeingPlaced = true;

      } else {
        // If No, we need to define random positions
        piecePosition = this_.setPiecesPosition();

        // Add pieces information in the Config.PUZZLES
        Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces.push({
          filename: 'piece-' + i,
          placed: false,
          actualPosition: [parseInt(piecePosition.top, 10), parseInt(piecePosition.left, 10)]
        })
      }

      // Create piece actor
      currentTempPiece = new PuzzleGame.Actor({
        filename:       'piece-' + i,
        filenameHover:  'piece-' + i,
        path:           path,
        styles:         piecePosition,
        mousedown:      function(current) { this_.currentPiece = current; }
      });

      // Make sure that the puzzle is not already completed
      if (Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].isDone != true) {
        // Make the piece draggable over the stage
        new Draggable($(currentTempPiece.element), this_.draggableOptions);
      }

      // Add the current piece in a_puzzlePiece's array
      this_.a_puzzlePiece.push(currentTempPiece);

    }

    // Wait for everybody on stage to be loaded and then call initDetectCollision
    PuzzleGame.OnloadObserver.queueHandler((function() {
      this_.initDetectCollision();
    }).bind(this));
  },

  // =============================================
  // initDetectCollision
  initDetectCollision: function() {
    var this_ = this;

    var newPosition = null;

    // If pieceBox property is not defined
    if (Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieceBox == undefined) {
      setTimeout(function() {

        // Loop on the number of pieces
        for (var i = 1, l = this_.a_puzzlePiece.length; i <= l; i++) {

          // Detect if there's any collision
          while (this_.detectCollision(this_.a_puzzlePiece[i - 1])) {
            // Affect the random position to a variable so we can reuse it further in the function
            newPosition = this_.setPiecesPosition();

            // Set new positions to the currentPiece
            this_.a_puzzlePiece[i - 1].setStyles(newPosition);

            // Change the actual position of the current piece for new values
            Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces[i - 1].actualPosition = [parseInt(newPosition.top), parseInt(newPosition.left)];
          }
        }
      }, 200)
    }
  },

// =============================================
  // detectCollision
  // @ Params
  //  - currentPiece = object
  detectCollision: function(currentPiece) {
    var this_ = this;

    var offset                  = {},
        dimensions              = {};

    var currentPieceOffset      = $(currentPiece.element).position(),
        currentPieceDimensions  = $(currentPiece.element).size();

    var watermarkOffset         = $(this.watermark.element).position(),
        watermarkDimensions     = $(this.watermark.element).size();

    // Verify if the current piece is over the watermark
    if (
      (currentPieceOffset.x + currentPieceDimensions.x) >= watermarkOffset.x && currentPieceOffset.x <= (watermarkOffset.x + watermarkDimensions.x - Config.OVERLAP[PuzzleGame.currentSize][game.currentPuzzlePieceCount].watermark) &&
      (currentPieceOffset.y + currentPieceDimensions.y) >= watermarkOffset.y && currentPieceOffset.y <= (watermarkOffset.y + watermarkDimensions.y - Config.OVERLAP[PuzzleGame.currentSize][game.currentPuzzlePieceCount].watermark)
    ) {
      return true;
    }

    // Pieces are placed
    this_.arePiecesDoneBeingPlaced = true;

    // Loop on every pieces
    for (var i = 0, l = this.a_puzzlePiece.length; i < l; i++) {

      // If current i index is the same as currentPiece
      if (this.a_puzzlePiece[i] == currentPiece) {
        continue;
      }

      offset      = $(this.a_puzzlePiece[i].element).position();
      dimensions  = $(this.a_puzzlePiece[i].element).size();

      // If currentPiece is over an other piece
      if (
        (currentPieceOffset.x + currentPieceDimensions.x) >= offset.x && currentPieceOffset.x <= (offset.x + dimensions.x - Config.OVERLAP[PuzzleGame.currentSize][game.currentPuzzlePieceCount].piece) &&
        (currentPieceOffset.y + currentPieceDimensions.y) >= offset.y && currentPieceOffset.y <= (offset.y + dimensions.y - Config.OVERLAP[PuzzleGame.currentSize][game.currentPuzzlePieceCount].piece)
      ) {
        return true;
      }
    }


    return false;
  },

   // =============================================
  // detectIfPieceIsNearTarget
  detectIfPieceIsNearTarget: function() {
    var this_ = this;

    var currentPieceOffsetLeft  = this_.currentPiece.element.offsetLeft,
        currentPieceOffsetTop   = this_.currentPiece.element.offsetTop;

    var currentId               = this_.currentPiece.id.replace('piece-', '');

    var target                  = Config.TARGET_POSITION[game.currentPuzzlePieceCount][PuzzleGame.currentSize][currentId - 1],
        currentConfigPiece      = Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces[currentId - 1];

    // Look if the pieces is near the top/left corner of the target position within the radius
    if (
      (currentPieceOffsetTop < (target.top + Config.RADIUS)) && (currentPieceOffsetTop > (target.top - Config.RADIUS)) &&
      (currentPieceOffsetLeft < (target.left + Config.RADIUS)) && (currentPieceOffsetLeft > (target.left - Config.RADIUS))
    ) {

      // You've made it ! Piece is close enough of top/left corner.

      // Set the actual position of the current piece with the target destination
      currentConfigPiece.actualPosition = [target.top, target.left];

      // We move the place into good position
      $(this_.currentPiece.element).morph({
        top: target.top + 'px',
        left: target.left + 'px'
      });

      // Play a sound to confirm the good placement
      game.soundEffect.play('place');

      // Set placed property to true so the piece knows it is well placed
      currentConfigPiece.placed = true;

      // Remove draggable options
      $(this_.currentPiece.element).undoDraggable();

      // Verify how many pieces are well placed
      this_.verifyIfCurrentPuzzleIsDone();

    } else {
      // The piece is not close enough of the top/left corner

      // Set placed property to false because
      currentConfigPiece.placed = false;

      // Animate the piece going back to original place
      $(this_.currentPiece.element).morph({
        top: this_.currentPiecePositionOnBefore[0] + 'px',
        left: this_.currentPiecePositionOnBefore[1] + 'px'
      }, 400);

      game.soundEffect.play('undo');
    }

    // Change the z-index value of the current piece that has been dragged
    if (this_.currentPiece != null && this_.currentPiece != undefined) {
      $(this_.currentPiece.element).setStyle('z-index', '403');
    }
  },

  // =============================================
  // showClue
  showClue: function() {
    var this_ = this;

    this.clue = new PuzzleGame.Actor({
      filename: 'clue-' + this_.currentPuzzleId,
      extension: 'png',
      path: 'assets/images/backgrounds/' + game.currentPuzzlePieceCount,
      styles: {
        zIndex: 401,
        left: this_.cluePosition.left,
        bottom: this_.cluePosition.bottom
      },
      dimensions: PuzzleGame.dimensions,
      interactive: false
    })
  },

  // =============================================
  // hideClue
  hideClue: function() {
    this.clue.destroy();
  },

  // =============================================
  // setPiecesPosition
  setPiecesPosition: function() {
    var this_ = this;

    // Get the actual dimensions of background
    var actualHeight  = PuzzleGame.dimensions[PuzzleGame.currentSize].height - this_.a_stagePadding[0],
        actualWidth   = PuzzleGame.dimensions[PuzzleGame.currentSize].width - this_.a_stagePadding[1];

    // Generate random values using the scene dimensions
    var xRand = Math.random(actualWidth),
        yRand = Math.random(actualHeight) + 25;

    return { zIndex: 403, left: xRand + 'px', top: yRand + 'px' };
  },

  // =============================================
  // resetPuzzle
  resetPuzzle: function() {
    var this_ = this;
    var currentConfigPiece, calculatedRandomPosition;

    // Remove the clue if visible
    if (this.clue != undefined) {
      this_.hideClue();
    }

    switch(game.currentPuzzlePieceCount) {
      case '12':
        Config.filename_12_completed = '';
        break;

      case '16':
        Config.filename_16_completed = '';
        break;

      case '20':
        Config.filename_20_completed = '';
        break;
    }

    // Reset each pieces randomly. It doesn't replace them at the initial place.
    for (var i = 0, l = game.currentPuzzlePieceCount; i < l; i++) {
      currentConfigPiece        = Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces[i];
      calculatedRandomPosition  = this_.setPiecesPosition();

      // We set the new positions and add them inside the PUZZLES object
      this_.a_puzzlePiece[i].setStyles(this_.setPiecesPosition());

      // Add draggable options to the current piece (make sure we can move it again)
      $(this_.a_puzzlePiece[i].element).makeDraggable(this_.draggableOptions);

      // Change the actual position of each piece
      Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces[i].actualPosition = [parseInt(calculatedRandomPosition.top, 10), parseInt(calculatedRandomPosition.left, 10)];

      // Reset properties
      Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces[i].placed = false;
      Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].isDone = false;

      while (this_.detectCollision(this_.a_puzzlePiece[i])) {
        calculatedRandomPosition = this_.setPiecesPosition();

        // We do the same thing as we did before the while loop
        // But this time, we check for collisions
        this_.a_puzzlePiece[i].setStyles(calculatedRandomPosition);
        Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces[i].actualPosition = [parseInt(calculatedRandomPosition.top, 10), parseInt(calculatedRandomPosition.left, 10)];
      }
    }
  },

  // =============================================
  // switchPuzzle
  // @ Params
  //  - direction = number
  switchPuzzle: function(direction) {
    var this_ = this;

    this_.arePiecesDoneBeingPlaced = false;

    // Set the array that contains all pieces inside pieceBox so when we come back on the same puzzle, so we know where pieces were
    Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieceBox = this_.a_puzzlePiece;

    game.soundEffect.stop();

    // (In/De)crement the currentPuzzleId with the direction
    this_.currentPuzzleId = this_.currentPuzzleId + direction;

    // Is currentPuzzleId value lower than 1?
    if (this_.currentPuzzleId < 1) {
      // Yeah, then get the last puzzle of the set
      this_.currentPuzzleId = parseInt(Config.PUZZLES[game.currentPuzzlePieceCount].length, 10);
    } else {
      // Nop, so let's make sure that currentPuzzleId is not higher than the number of puzzles of the same size
      if (this_.currentPuzzleId > parseInt(Config.PUZZLES[game.currentPuzzlePieceCount].length, 10)) {
        // If yes, then set currentPuzzleId to 1
        this_.currentPuzzleId = 1;
      }
    }

    // Delete puzzle pieces on stage
    game.puzzle.a_puzzlePiece.each('destroy');

    // Reset to empty array all pieces in a_puzzlePiece
    this_.a_puzzlePiece = [];

    // Reset currentPiece
    this_.currentPiece = null;

    // Remove the clue if visible
    if (this.clue != undefined) {
      this_.hideClue();
    }

    // Re-set the puzzles pieces on stage
    this_.initPieces();

  },
  // }}}

  // =============================================
  // verifyIfCurrentPuzzleIsDone
  verifyIfCurrentPuzzleIsDone: function() {
    var this_ = this;
    var numberOfPiecesWellPlaced = 0;

    // Loop on every pieces to verify if everyone is well placed
    for (var i = 0, l = game.currentPuzzlePieceCount; i < l; i++) {

      // If placed property is set to false, we continue to the other one
      if (Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieces[i].placed == false) {
        continue;
      }

      numberOfPiecesWellPlaced += 1;

      // Verify if the howManyCompleted variable has reached the same number as the current puzzle piece count
      // It means everything is in place ! The current puzzle is finished
      if (numberOfPiecesWellPlaced == parseInt(game.currentPuzzlePieceCount, 10)) {
        // Show the clue
        this_.showClue();

        // Play the sound associated to the puzzle
        game.soundEffect.play(Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].sound);

        // Set isDone property to true
        Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].isDone = true;

        // Verify if all puzzles in current size are done
        this_.verifyIfPuzzlesInCurrentSizeAreDone();

        // this_.switchPuzzle(1);
      }
    }
  },

  // =============================================
  // verifyIfPuzzlesInCurrentSizeAreDone
  verifyIfPuzzlesInCurrentSizeAreDone: function() {
    var this_ = this;

    var numberOfPuzzlesDone = 0;

    // Loop on every puzzles
    for (var i = 0, l = Config.PUZZLES[game.currentPuzzlePieceCount].length; i < l; i++) {

      // Is isDone property set to true?
      if (Config.PUZZLES[game.currentPuzzlePieceCount][i].isDone == true) {
        numberOfPuzzlesDone += 1;
      }
    }

    if (numberOfPuzzlesDone == Config.PUZZLES[game.currentPuzzlePieceCount].length) {
      // Show the confirmation window for all puzzles in set
      setTimeout(function(){
        game.showCompletedPuzzleSetWindow();

        // Add a timeout to go back to home screen
        setTimeout(function() {

          // Destroy the current view
          game.destroy();

          // Go back to home screen and change the buttons
          game.setupHomeScreen(game.currentPuzzlePieceCount);

          // Clear the array a_puzzlePiece
          game.puzzle.a_puzzlePiece.each('destroy');

          // Close the dialog window
          game.hideDialog();

          // Set pieceBox property
          Config.PUZZLES[game.currentPuzzlePieceCount][this_.currentPuzzleId - 1].pieceBox = game.puzzle.a_puzzlePiece;

          // Hide the clue
          this_.hideClue();
        }, Config.TIME_BEFORE_GOING_HOME_SCREEN);
      }, Config.TIME_BEFORE_DIALOG);

    } else {
      // Show the confirmation window for one puzzle
      // Add timeout to give time for the sound
      setTimeout(function(){
        game.showCompletedPuzzleWindow();
      }, Config.TIME_BEFORE_GREEN_SCREEN);
    }
  }
});
